@@ -12,7 +12,6 @@
 
     TODO:
     * roundtrip the question in some other way:
-     * use safe encoding / encryption for the q
      * make sure a q/a pair in the POST is for the q in the GET before
     * make some nice CSS
     * make similar changes to GUI editor
@@ -24,10 +23,16 @@
 import re
 import random
 
+from time import time
+
 from MoinMoin import log
 logging = log.getLogger(__name__)
 
 from MoinMoin import wikiutil
+from MoinMoin.support.python_compatibility import hmac_new
+
+SHA1_LEN = 40 # length of hexdigest
+TIMESTAMP_LEN = 10 # length of timestamp
 
 class TextCha(object):
     """ Text CAPTCHA support """
@@ -41,15 +46,19 @@ class TextCha(object):
         self.request = request
         self.user_info = request.user.valid and request.user.name or request.remote_addr
         self.textchas = self._get_textchas()
+        if self.textchas:
+            self.secret = request.cfg.secrets["security/textcha"]
+            self.expiry_time = request.cfg.textchas_expiry_time
         self._init_qa(question)
 
     def _get_textchas(self):
         """ get textchas from the wiki config for the user's language (or default_language or en) """
         request = self.request
+        groups = request.groups
         cfg = request.cfg
         user = request.user
         disabled_group = cfg.textchas_disabled_group
-        if disabled_group and user.name and request.dicts.has_member(disabled_group, user.name):
+        if disabled_group and user.name and user.name in groups.get(disabled_group, []):
             return None
         textchas = cfg.textchas
         if textchas:
@@ -73,6 +82,10 @@ class TextCha(object):
             logging.debug(u"TextCha: using lang = '%s'" % lang)
             return textchas[lang]
 
+    def _compute_signature(self, question, timestamp):
+        signature = u"%s%d" % (question, timestamp)
+        return hmac_new(self.secret, signature.encode('utf-8')).hexdigest()
+
     def _init_qa(self, question=None):
         """ Initialize the question / answer.
 
@@ -89,8 +102,8 @@ class TextCha(object):
                 self.answer_re = re.compile(self.answer_regex, re.U|re.I)
             except KeyError:
                 # this question does not exist, thus there is no answer
-                self.answer_regex = ur"[^.]*" # this shall never match!
-                self.answer_re = re.compile(self.answer_regex, re.U|re.I)
+                self.answer_regex = ur"[Never match for cheaters]"
+                self.answer_re = None
                 logging.warning(u"TextCha: Non-existing question '%s'. User '%s' trying to cheat?" % (
                                 self.question, self.user_info))
             except re.error:
@@ -113,10 +126,22 @@ class TextCha(object):
         """
         return not not self.textchas # we don't want to return the dict
 
-    def check_answer(self, given_answer):
-        """ check if the given answer to the question is correct """
+    def check_answer(self, given_answer, timestamp, signature):
+        """ check if the given answer to the question is correct and within the correct timeframe"""
         if self.is_enabled():
-            success = self.answer_re.match(given_answer.strip()) is not None
+            if self.answer_re is not None:
+                success = self.answer_re.match(given_answer.strip()) is not None
+            else:
+                # someone trying to cheat!?
+                success = False
+            if not timestamp or timestamp + self.expiry_time < time():
+                success = False
+            try:
+                if self._compute_signature(self.question, timestamp) != signature:
+                    success = False
+            except TypeError:
+                success = False
+
             success_status = success and u"success" or u"failure"
             logging.info(u"TextCha: %s (u='%s', a='%s', re='%s', q='%s')" % (
                              success_status,
@@ -130,16 +155,36 @@ class TextCha(object):
             return True
 
     def _make_form_values(self, question, given_answer):
-        question_form = wikiutil.escape(question, True)
+        timestamp = time()
+        question_form = "%s %d%s" % (
+            wikiutil.escape(question, True),
+            timestamp,
+            self._compute_signature(question, timestamp)
+        )
         given_answer_form = wikiutil.escape(given_answer, True)
         return question_form, given_answer_form
 
     def _extract_form_values(self, form=None):
         if form is None:
             form = self.request.form
-        question = form.get('textcha-question', [None])[0]
-        given_answer = form.get('textcha-answer', [u''])[0]
-        return question, given_answer
+        question = form.get('textcha-question')
+        signature = None
+        timestamp = None
+        if question:
+            # the signature is the last SHA1_LEN bytes of the question
+            signature = question[-SHA1_LEN:]
+
+            # operate on the remainder
+            question = question[:-SHA1_LEN]
+            try:
+                # the timestamp is the next TIMESTAMP_LEN bytes
+                timestamp = int(question[-TIMESTAMP_LEN:])
+            except ValueError:
+                pass
+            # there is a space between the timestamp and the question, so take away 1
+            question = question[:-TIMESTAMP_LEN - 1]
+        given_answer = form.get('textcha-answer', u'')
+        return question, given_answer, timestamp, signature
 
     def render(self, form=None):
         """ Checks if textchas are enabled and returns HTML for one,
@@ -148,7 +193,7 @@ class TextCha(object):
             @return: unicode result html
         """
         if self.is_enabled():
-            question, given_answer = self._extract_form_values(form)
+            question, given_answer, timestamp, signature = self._extract_form_values(form)
             if question is None:
                 question = self.question
             question_form, given_answer_form = self._make_form_values(question, given_answer)
@@ -165,9 +210,9 @@ class TextCha(object):
 
     def check_answer_from_form(self, form=None):
         if self.is_enabled():
-            question, given_answer = self._extract_form_values(form)
+            question, given_answer, timestamp, signature = self._extract_form_values(form)
             self._init_qa(question)
-            return self.check_answer(given_answer)
+            return self.check_answer(given_answer, timestamp, signature)
         else:
             return True
 
