@@ -869,8 +869,10 @@ var search_hint = "%(search_hint)s";
         """
         if not rss_supported:
             return False
-        return page.page_name == u'RecentChanges' or \
-           page.page_name == self.request.getText(u'RecentChanges')
+        return page.page_name in [u'RecentChanges',
+            self.request.getText(u'RecentChanges'),
+            self.request.cfg.page_front_page,
+            self.request.getText(self.request.cfg.page_front_page)]
 
     def rsshref(self, page):
         """ Create rss href, used for rss button and head link
@@ -899,6 +901,14 @@ var search_hint = "%(search_hint)s";
                     u'href="%s" type="application/rss+xml">') % (
                         wikiutil.escape(self.cfg.sitename, True),
                         wikiutil.escape(self.rsshref(page), True) )
+        elif rss_supported and self.cfg.rss_show_page_history_link:
+            link = (u'<link rel="alternate" title="%s: %s" '
+                    u'href="%s" type="application/rss+xml">') % (
+                        wikiutil.escape(self.cfg.sitename, True), page.page_name,
+                        wikiutil.escape(page.url(self.request, querystr={
+                            'action': 'rss_rc', 'ddiffs': '1', 'unique': '0',
+                            'diffs': '1', 'show_att': '1',
+                            'page': page.page_name }, escape=0), True) )
         return link
 
     def html_head(self, d):
@@ -1621,6 +1631,9 @@ var gui_editor_link_text = "%(text)s";
         home_page = wikiutil.getInterwikiHomePage(request) # sorry theme API change!!! Either None or tuple (wikiname,pagename) now.
         page_parent_page = getattr(page.getParentPage(), 'page_name', None)
 
+        # set content_type, including charset, so web server doesn't touch it:
+        request.content_type = "text/html; charset=%s" % (config.charset, )
+
         # Prepare the HTML <head> element
         user_head = [request.cfg.html_head]
 
@@ -1635,6 +1648,13 @@ var gui_editor_link_text = "%(text)s";
         if meta_desc:
             user_head.append('<meta name="description" content="%s">\n' % wikiutil.escape(meta_desc, 1))
 
+        #  add meta statement if user has doubleclick on edit turned on or it is default
+        if (pagename and keywords.get('allow_doubleclick', 0) and
+            not keywords.get('print_mode', 0) and
+            request.user.edit_on_doubleclick):
+            if request.user.may.write(pagename): # separating this gains speed
+                user_head.append('<meta name="edit_on_doubleclick" content="%s">\n' % (request.script_root or '/'))
+
         # search engine precautions / optimization:
         # if it is an action or edit/search, send query headers (noindex,nofollow):
         if request.query_string:
@@ -1738,14 +1758,6 @@ var gui_editor_link_text = "%(text)s";
             bodyattr.append(' ')
             bodyattr.append(keywords['body_attr'])
 
-        # Add doubleclick edit action
-        if (pagename and keywords.get('allow_doubleclick', 0) and
-            not keywords.get('print_mode', 0) and
-            request.user.edit_on_doubleclick):
-            if request.user.may.write(pagename): # separating this gains speed
-                url = page.url(request, {'action': 'edit'})
-                bodyattr.append(''' ondblclick="location.href='%s'" ''' % wikiutil.escape(url, True))
-
         # Set body to the user interface language and direction
         bodyattr.append(' %s' % self.ui_lang_attr())
 
